###############################################################################################
# $Id: scenario_manager.py,v 1.40 2003/03/06 14:08:40 chakie Exp $
###############################################################################################

import os, sys, string
import os.path
import pyexpat
import glob
import zipfile
import cPickle

import scenario
import properties
from simple_dom_parser import SimpleDOMParser, Node
from scenario_info     import ScenarioInfo
from constants         import REBEL, UNION

class ScenarioManager:
    """
    This class works as a base class for managing scenarios in Civil. The subclasses
    ClientScenarioManager and ServerScenarioManager are those that should be instantiated by the
    client and server. This class contains common functionality needed by both classes.
    """

    def __init__ (self):
        """Intializes the instance."""
        # init all members
        self.scenarios = []
        
    
    def getScenarios (self):
        """Returns all the parsed scenario infos. The returned list contains instance of
        ScenarioInfo that describe all the scenarios that are available. If something has failed or
        there are no scenarios available the list is empty."""
        return self.scenarios
    

    def parseScenarioInfo (self, root, id):
        """Parses the <scenarioinfo> tag, extracts and stores all the info in a ScenarioInfo
        object. The passed 'root' is expected to be the <scenarioinfo> node. Returns the info object
        if all is ok and None on error."""
        
        # is it a proper info node?
        if root.getName () != 'scenarioinfo':
            # oops, illegal file
            print "DOM tree is illegal, toplevel tag is: " + root.getName () + ", "
            print "the node should be 'scenarioinfo'"
            return None

        try:
            # get the simple data
            name        = root.getChild ('name').getData ()
            location    = root.getChild ('location').getData ()
            maxturns    = int ( root.getChild ('turns').getAttribute ( 'max' ) )
            currentturn = int ( root.getChild ('turns').getAttribute ( 'current' ) )
 
            # get the file if we have it
            file = root.getChild ('file')
                
            # get the date and it
            datenode = root.getChild ('date')
            year     = int ( datenode.getAttributes ()['year'] )
            month    = int ( datenode.getAttributes ()['month'] )
            day      = int ( datenode.getAttributes ()['day'] )
            hour     = int ( datenode.getAttributes ()['hour'] )
            minute   = int ( datenode.getAttributes ()['minute'] )

            # no description yet
            description = []
            
            # and the description
            for paratag in root.getChild ('description').getChildren ():
                # get the text
                para = paratag.getData ()
                
                # append the text to the description
                description.append ( para )

            # get the rebel and union missions
            rebel = []
            union = []

            # get the missions
            mission = root.getChild ('missions')
            
            # rebel misisons
            for paratag in mission.getChild ( 'rebel').getChildren ():
                # get the text and append to the description
                rebel.append ( paratag.getData () )

            # union misisons
            for paratag in mission.getChild ( 'union').getChildren ():
                # get the text and append to the description
                union.append ( paratag.getData () )
            
            # create a new scenario info and set all parsed data
            info = ScenarioInfo ( id )
            info.setName ( name )
            info.setLocation ( location )
            info.setMaxTurns ( maxturns )
            info.setCurrentTurn ( currentturn )
            info.setDescription ( description )
            info.setMission ( REBEL, rebel )
            info.setMission ( UNION, union )
            info.setDate ( (year, month, day, hour, minute ) )

            # now we need to see if the scenario is valid at all
            if root.getChild ('valid') == None:
                # no such node, so the scenario is invalid
                info.setValid ( 0 )
            else:
                # it's valid all righ
                info.setValid ( 1 )
                
            # add to our internal list
            self.scenarios.append ( info )
            
            # all ok
            return info
        
        except KeyError:
            # oops, invalid data
            print "Invalid scenario info."
            return None
    

##################################################################################################

class ClientScenarioManager (ScenarioManager):
    """This class subclasses the scenario manager and adds methods needed by the clients to manage
    scenarios. The methods are those used to communicate with the server and request/retrieve
    scenarios. """
    

    def __init__ (self):
        """Initializes the instance."""
        # call superclass
        ScenarioManager.__init__ (self )

        
    def parseScenarioIndex (self, root):
        """Parses a passed DOM-tree and builds up a number of instances of ScenarioInfo. They are
        appended to the internal list of senario-info:s. Returns 1 if all is ok and the entrie index
        file was parsed, and 0 on error."""

        # is it a proper index file?
        if root.getName () != 'scenarioindex':
            # oops, illegal file
            print "DOM tree is illegal, toplevel tag is: " + root.getName () + ", "
            print "the node should be 'scenarioindex'"
            return 0
        
        # loop over all scenarioinfo:s
        for info in root.getChildren ():

            # parse the current info
            if not self.parseScenarioInfo ( info ):
                # parsing failed
                print "parsing failed"
                return 0

        # all is ok
        return 1
    
    
    def retrieveScenarios (self, connection):
        """Reads scenarios from the server over open connection. Will request the server to send all
        the available scenarios to us. Not the full scenarios are sent, only the info part
        (header). This information is all the player needs to be able to decide upon which scenario
        to use."""

        # request the scenarios
        connection.send ( 'get_scenarios\n' )
        
        try:
            # read the number of lines, wait for 5 seconds
            line = connection.readLine ( timeout = 5 )

            # did we get a line?
            if not line:
                print "ScenarioManager.retrieveScenarios: error or timeout receiving line count"
                return 0
           
            # cast the data
            count = int ( string.strip (line ) )
            xml_data = []
            
            # loop and read the lines
            for index in range ( count ):
                # read a line
                line = connection.readLine ( timeout = 5 )

                # did we get a line?
                if not line:
                    continue
                
                # add to the internal list
                xml_data.append ( line )

            # join the data
            buffer = ''
            buffer = '\n'.join ( xml_data )

            # parse the read data
            root = SimpleDOMParser ().parseString ( buffer )

        except:
            # serios error
            print "ScenarioManager.retrieveScenarios: error retrieving scenarios"
            raise
            return 0
                
        # did we get anything?
        if root == None:
            # nope, failed to retrieve it
            print "ScenarioManager.retrieveScenarios: failed to retrive scenarios"
            return 0

        # now parse the data
        self.parseScenarioIndex ( root )


    def retrieveFullScenario (self, connection, id):
        """Retrieves the full scenario 'id' from the server. Builds up a buffer with the data, but
        does not actually parse it in any way. Returns the buffer, or None on error."""

        # request the scenario
        connection.send ( 'scenario %d\n' % id )

        # no data yet
        buffer = ''
       
        try:
            # read the number of lines, wait for 5 seconds
            line = connection.readLine ( timeout = 5 )

            # did we get a line?
            if not line:
                print "ClientScenarioManager.retrieveFullScenario: error or timeout receiving line count"
                return 0
           
            # cast the data
            count = int ( string.strip (line ) )
            
            # loop and read the lines
            for index in range ( count ):
                # read a line
                line = connection.readLine ( timeout = 5 )

                # did we get a line?
                if not line:
                    continue
                
                # add to the internal list
                buffer += line

            # all is ok, return the buffer
            return buffer
 
        except:
            # serios error
            print "ClientScenarioManager.retrieveFullScenario: error retrieving scenario"
            raise
            return None


    def readScenario (self, connection, count, progress_callback=None):
        """Reads one scenario from 'connection'. This differs from the above the method
        retrieveFullScenario() as it does not first request the scenario, instead it assumes a
        scenario is already coming down the pipe. The scenario must be 'count' lines of
        data. Returns the buffer, or None on error.

        If the callback 'progress_callback' is not None, then it is called for every line of the
        scenario. It can perform some kind of progress visualization to the player. The callback
        should accept two parameters, namely the current line and the total number of lines. If it
        is None it is ignored.  
        """ 

        buffer = ""

        try:
            # loop and read the lines
            for index in range ( count ):
                # read a line
                line = connection.readLine ( timeout = 5 )

                # did we get a line?
                if not line:
                    continue
                
                # add to the internal list
                buffer += line

                # do the progress callback
                if progress_callback != None:
                    # call it
                    progress_callback ( index, count )
                    
            # all is ok, return the buffer
            return buffer
 
        except:
            # serios error
            print "ClientScenarioManager.readScenario: error retrieving scenario"
            raise
            return None


    def readLosMap (self, connection, los_width, los_height, progress_callback=None):
        """Reads the LOS map from the connection and returns it. Calls the progress every 100 values."""

        print "readLosMap: reading %d x %d" % (los_width, los_height)
        
        # initialize the los map to an array of the proper height. the contents for the rows is
        # created later
        los_map = [ 0 ] * los_height

        count = 0
        total = los_height
        
        # loop over the entire los map matrix 
        for y in range ( los_height ):
            # read a whole line of data, this is values separated by spaces
            line = connection.readLine ( timeout=60 )
            
            # convert the line of values to a list of numbers
            los_map [y] = map ( int, line.split () )

            
            # time for a progress update?
            if progress_callback != None:
                # yep, call the progress callback
                progress_callback ( count, total )

            # one more line read
            count += 1
            
        return los_map
    

##################################################################################################

class ServerScenarioManager (ScenarioManager):
    
    """This class subclasses the scenario manager and adds methods needed by the server to manage
    scenarios. It mainly knows where to find information about all available scenarios and can then
    parse a wanted scenario file. Scenarios are quite big, so separate information about them is
    kept in an index files. This class knows where all the index files are, reads and parses them
    into a list with instances of ScenarioInfo. This list can then be accessed by external classes
    and the user can be presented with a list of all scenarios.

    The client class ClientScenarioManager will request scenarioinfos and scenarios from this
    class.

    This class can also write out a new scenario index file from all loaded scenarios.
    """

    def __init__ (self):
        """Initializes the instance and load the scenario index file."""

        # call superclass
        ScenarioManager.__init__ (self )

        # a map from id:s to scenario filenames
        self.filenames = {}

        
    def sendScenarioInfo (self, connection):
        """Sends the raw ASCII XML data to the client behind 'connection'. First the number of lines
        is sent and then the raw data one line at a time."""

        # build up the raw xml we're about to send
        xml = [ '<?xml version="1.0"?><scenarioindex>\n' ]

        # loop over all infos we got
        for info in self.scenarios:
            # add the xml for this info
            for line in info.toXML ().split ( '\n' ):
                xml.append ( line + '\n' )

        # append the closing tag
        xml.append ( '</scenarioindex>\n' )
        
        # get the number of lines
        count = str ( len ( xml ) ) + '\n'

        # send the number of lines
        connection.send ( count )
        
        # now loop over all lines
        for line in xml:
            # send it
            connection.send ( line )


    def sendScenario (self, connection, id):
        """Sends the scenario with the given id to the client to which 'connection' leads. First
        the number of lines, the id for the remote player and the name of the local player is sent
        and then the lines one at a time.""" 

        # map the id to a filename
        if not self.filenames.has_key ( id ):
            # no matching scenario found
            raise "invalid scenario id: " + str(id)

        # get the filename
        filename = self.filenames [id]

        # does the file exists?
        if not os.path.exists ( filename ):
            # no such file
            print "could not find scenario:", filename
            return

        # open the file as a zip file
        archive = zipfile.ZipFile ( filename )

        # get the main scenario file from the archive
        data = archive.read ( 'scenario.xml' )

        # and close it
        archive.close ()

        # get the los map
        los_map    = scenario.map.getLosMap ()
        los_width  = len (los_map[0])
        los_height = len (los_map)
        total      = los_height * los_width

        # create the info
        info = '%d %d %d %d %s\n' % ( (UNION, REBEL)[scenario.local_player_id],
                                      data.count ( '\n' ), los_width, los_height,
                                      scenario.local_player_name )
        
        # send the basic data
        connection.send ( info )
        
        # send the scenario too
        connection.send ( data )

        print "sendScenario: sent:", info
        print "sendScenario: sending %d x %d" % (los_width, los_height)

        # now loop over the entire los map matrix 
        for y in range ( los_height ):
            # clear this line
            line = ''
            
            for x in range ( los_width ):
                # add the integer as a string
                line += str ( los_map [y][x] ) + " "

            # ok, we have a full line now, terminate it
            line += '\n'

            # send it away
            connection.send ( line )

            #print "sendScenario: sent:", line
            
        # we're done here
        return


    def loadScenario (self, id):
        """Loads the scenario with the given id. Opens the scenario archive file and reads the
        scenario and LOS data. The scenario is returned as a string and the LOS data as a pickled
        object. If the scenario contains no LOS map then None is returned for that part. On error
        None is returned for both objects.""" 

        # map the id to a filename
        if not self.filenames.has_key ( id ):
            # no such id?
            print "invalid scenario id:", id
            return None, None

        # get the filename of the scenario
        filename = self.filenames [id]

        # does the file exists?
        if not os.path.exists ( filename ):
            # no such file
            print "could not find scenario:", filename
            return None, None

        print "loading scenario: %s" % filename

        # open the file as a zip file
        archive = zipfile.ZipFile ( filename )

        # get the main scenario file from the archive
        scenario = archive.read ( 'scenario.xml' )

        # the los map *may* not be there. check if it is
        if 'los.pickle' in archive.namelist ():
            # yes, we have it there, read it
            los_map = archive.read ( 'los.pickle' )
        else:
            # no los map
            los_map = None
            
        # and close it
        archive.close ()
        
        # we're done here
        return scenario, los_map


    def loadScenarioInfo (self, filename, id):
        """This method reads scenario info from the passed scenario file. The file is opened as a
        zip file and the 'info.xml' part is extracted and parsed. The info is given 'id' as an
        id. The final info data is stored internally and associated with the file name. Returns 1 if
        all is ok and 0 on error.""" 

        try:
            # open the file as a zip file
            archive = zipfile.ZipFile ( filename )

            # get the info file from the archive
            buffer = archive.read ( 'info.xml' )
        except:
            # not a scenario?
            print "ServerScenarioManager.loadScenarioInfo: invalid archive:", filename
            return 0

        # create a parser
        parser = SimpleDOMParser ()
            
        # parse the read data
        root = parser.parseString ( buffer )
                
        # did we get anything?
        if root == None:
            # nope, failed to retrieve it
            print "ServerScenarioManager.loadScenarioInfo: parse error for:", filename
            archive.close ()
            return 0

        # now parse the data
        info = self.parseScenarioInfo ( root, id )

        # did we do ok?
        if info != None:
            # yes, all ok, store an entry in the name map so that we can later identify the file for
            # the info based on its id
            self.filenames [ id ] = filename
            
        # close the archive too
        archive.close ()

        # return whatever we got
        return info


    def loadAllScenarioInfos (self, dir):
        """Attempts to load all info files for scenarios that are found in 'dir'. Loads scenario
        infos from the files and stores them internally. Returns 1 if all is ok, and 0 if not."""

        # start from id 0
        id = 0
        
        # loop over all files in the directory that are civil scenarios
        for file in glob.glob ( dir + "/*.civil" ):
            print "Parsing scenario: %s..." % file

            # load and parse it
            if not self.loadScenarioInfo ( file, id ):
                # we failed to read it, so abort all reading
                return 0

            # next id
            id += 1
            
        # we got this far, so all must be ok
        return 1


#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
